@@ -117,6 +117,9 @@ class ClassDef : public Definition
                         Singleton, //=Entry::CLASS_SEC
                       };
 
+    virtual std::unique_ptr<ClassDef> deepCopy(const QCString &name) const = 0;
+    virtual void moveTo(Definition *) = 0;
+
     //-----------------------------------------------------------------------------------
     // --- getters
     //-----------------------------------------------------------------------------------
@@ -329,22 +332,16 @@ class ClassDef : public Definition
 
     virtual bool containsOverload(const MemberDef *md) const = 0;
 
+    virtual bool isImplicitTemplateInstance() const = 0;
+
     //-----------------------------------------------------------------------------------
     // --- count members ----
     //-----------------------------------------------------------------------------------
 
     virtual int countMembersIncludingGrouped(MemberListType lt,
                 const ClassDef *inheritedFrom,bool additional) const = 0;
-    virtual int countInheritanceNodes() const = 0;
     virtual int countMemberDeclarations(MemberListType lt,const ClassDef *inheritedFrom,
-                int lt2,bool invert,bool showAlways,ClassDefSet &visitedClasses) const = 0;
-
-    //-----------------------------------------------------------------------------------
-    // --- helpers ----
-    //-----------------------------------------------------------------------------------
-
-    virtual ClassDef *insertTemplateInstance(const QCString &fileName,int startLine,int startColumn,
-                                const QCString &templSpec,bool &freshInstance) const = 0;
+                MemberListType lt2,bool invert,bool showAlways,ClassDefSet &visitedClasses) const = 0;
 
     //-----------------------------------------------------------------------------------
     // --- write output ----
@@ -366,7 +363,7 @@ class ClassDef : public Definition
                  MemberListType lt,const QCString &title,
                  const QCString &subTitle=QCString(),
                  bool showInline=FALSE,const ClassDef *inheritedFrom=nullptr,
-                 int lt2=-1,bool invert=FALSE,bool showAlways=FALSE) const = 0;
+                 MemberListType lt2=MemberListType::Invalid(),bool invert=FALSE,bool showAlways=FALSE) const = 0;
     virtual void addGroupedInheritedMembers(OutputList &ol,MemberListType lt,
                  const ClassDef *inheritedFrom,const QCString &inheritId) const = 0;
 };
@@ -403,17 +400,26 @@ class ClassDefMutable : public DefinitionMutable, public ClassDef
         // inheritance graph related members
     virtual CLASS_GRAPH_t hasInheritanceGraph() const = 0;
     virtual void overrideInheritanceGraph(CLASS_GRAPH_t e) = 0;
+    virtual void setImplicitTemplateInstance(bool b) = 0;
 
     // collaboration graph related members
     virtual bool hasCollaborationGraph() const = 0;
     virtual void overrideCollaborationGraph(bool e) = 0;
 
+    //-----------------------------------------------------------------------------------
+    // --- helpers ----
+    //-----------------------------------------------------------------------------------
+
+    virtual ClassDef *insertTemplateInstance(const QCString &fileName,int startLine,int startColumn,
+                                const QCString &templSpec,bool &freshInstance) = 0;
+
     //-----------------------------------------------------------------------------------
     // --- actions ----
     //-----------------------------------------------------------------------------------
 
     virtual void insertBaseClass(ClassDef *,const QCString &name,Protection p,Specifier s,const QCString &t=QCString()) = 0;
     virtual void insertSubClass(ClassDef *,Protection p,Specifier s,const QCString &t=QCString()) = 0;
+    virtual void insertExplicitTemplateInstance(ClassDef *instance,const QCString &spec) = 0;
     virtual void insertMember(MemberDef *) = 0;
     virtual void insertUsedFile(const FileDef *) = 0;
     virtual void addMembersToTemplateInstance(const ClassDef *cd,const ArgumentList &templateArguments,const QCString &templSpec) = 0;
