@@ -80,44 +80,57 @@ inline void writeDocbookString(TextStream &t,const QCString &s)
   t << convertToDocBook(s);
 }
 
-inline void writeDocbookCodeString(TextStream &t,const QCString &str, size_t &col)
+inline void writeDocbookCodeString(bool hide,TextStream &t,const QCString &str, size_t &col, size_t stripIndentAmount)
 {
   if (str.isEmpty()) return;
+  const int tabSize = Config_getInt(TAB_SIZE);
   const char *s = str.data();
   char c=0;
-  while ((c=*s++))
+  if (hide)
   {
-    switch(c)
+    col = updateColumnCount(s,col);
+  }
+  else
+  {
+    while ((c=*s++))
     {
-      case '\t':
-        {
-          int tabSize = Config_getInt(TAB_SIZE);
-          int spacesToNextTabStop = tabSize - (col%tabSize);
-          col+=spacesToNextTabStop;
-          while (spacesToNextTabStop--) t << "&#32;";
-          break;
-        }
-      case ' ':  t << "&#32;"; col++;  break;
-      case '<':  t << "&lt;"; col++;   break;
-      case '>':  t << "&gt;"; col++;   break;
-      case '&':  t << "&amp;"; col++;  break;
-      case '\'': t << "&apos;"; col++; break;
-      case '"':  t << "&quot;"; col++; break;
-      default:
-        {
-          uint8_t uc = static_cast<uint8_t>(c);
-          static const char *hex="0123456789ABCDEF";
-          if (uc<32)
+      switch(c)
+      {
+        case '\t':
           {
-            t << "&#x24" << hex[uc>>4] << hex[uc&0xF] << ";";
-          }
-          else
-          {
-            t << c;
+            int spacesToNextTabStop = tabSize - (col%tabSize);
+            while (spacesToNextTabStop--)
+            {
+              if (col>=stripIndentAmount) t << "&#32;";
+              col++;
+            }
+            break;
           }
+        case ' ':
+          if (col>=stripIndentAmount) t << "&#32;";
           col++;
-        }
-        break;
+          break;
+        case '<':  t << "&lt;"; col++;   break;
+        case '>':  t << "&gt;"; col++;   break;
+        case '&':  t << "&amp;"; col++;  break;
+        case '\'': t << "&apos;"; col++; break;
+        case '"':  t << "&quot;"; col++; break;
+        default:
+                   {
+                     uint8_t uc = static_cast<uint8_t>(c);
+                     static const char *hex="0123456789ABCDEF";
+                     if (uc<32)
+                     {
+                       t << "&#x24" << hex[uc>>4] << hex[uc&0xF] << ";";
+                     }
+                     else
+                     {
+                       t << c;
+                     }
+                     col++;
+                   }
+                   break;
+      }
     }
   }
 }
@@ -153,7 +166,27 @@ DocbookCodeGenerator::DocbookCodeGenerator(TextStream *t) : m_t(t)
 void DocbookCodeGenerator::codify(const QCString &text)
 {
   Docbook_DB(("(codify \"%s\")\n",text));
-  writeDocbookCodeString(*m_t,text,m_col);
+  writeDocbookCodeString(m_hide,*m_t,text,m_col,static_cast<size_t>(m_stripIndentAmount));
+}
+
+void DocbookCodeGenerator::stripCodeComments(bool b)
+{
+  m_stripCodeComments = b;
+}
+
+void DocbookCodeGenerator::startSpecialComment()
+{
+  m_hide = m_stripCodeComments;
+}
+
+void DocbookCodeGenerator::endSpecialComment()
+{
+  m_hide = false;
+}
+
+void DocbookCodeGenerator::setStripIndentAmount(size_t amount)
+{
+  m_stripIndentAmount = amount;
 }
 
 void DocbookCodeGenerator::writeCodeLink(CodeSymbolType,
@@ -161,6 +194,7 @@ void DocbookCodeGenerator::writeCodeLink(CodeSymbolType,
     const QCString &anchor,const QCString &name,
     const QCString &tooltip)
 {
+  if (m_hide) return;
   Docbook_DB(("(writeCodeLink)\n"));
   writeDocbookLink(*m_t,ref,file,anchor,name,tooltip);
   m_col+=name.length();
@@ -171,6 +205,7 @@ void DocbookCodeGenerator::writeCodeLinkLine(CodeSymbolType,
     const QCString &,const QCString &name,
     const QCString &,bool writeLineAnchor)
 {
+  if (m_hide) return;
   Docbook_DB(("(writeCodeLinkLine)\n"));
   if (!writeLineAnchor) return;
   *m_t << "<anchor xml:id=\"_" << stripExtensionGeneral(stripPath(file),".xml");
@@ -189,6 +224,7 @@ void DocbookCodeGenerator::writeTooltip(const QCString &, const DocLinkInfo &, c
 
 void DocbookCodeGenerator::startCodeLine(int)
 {
+  if (m_hide) return;
   Docbook_DB(("(startCodeLine)\n"));
   m_insideCodeLine=TRUE;
   m_col=0;
@@ -196,6 +232,7 @@ void DocbookCodeGenerator::startCodeLine(int)
 
 void DocbookCodeGenerator::endCodeLine()
 {
+  if (m_hide) return;
   if (m_insideCodeLine) *m_t << "\n";
   Docbook_DB(("(endCodeLine)\n"));
   m_lineNumber = -1;
@@ -206,6 +243,7 @@ void DocbookCodeGenerator::endCodeLine()
 
 void DocbookCodeGenerator::startFontClass(const QCString &colorClass)
 {
+  if (m_hide) return;
   Docbook_DB(("(startFontClass)\n"));
   *m_t << "<emphasis role=\"" << colorClass << "\">";
   m_insideSpecialHL=TRUE;
@@ -213,6 +251,7 @@ void DocbookCodeGenerator::startFontClass(const QCString &colorClass)
 
 void DocbookCodeGenerator::endFontClass()
 {
+  if (m_hide) return;
   Docbook_DB(("(endFontClass)\n"));
   *m_t << "</emphasis>"; // non DocBook
   m_insideSpecialHL=FALSE;
@@ -226,6 +265,7 @@ void DocbookCodeGenerator::writeCodeAnchor(const QCString &)
 void DocbookCodeGenerator::writeLineNumber(const QCString &ref,const QCString &fileName,
     const QCString &anchor,int l,bool writeLineAnchor)
 {
+  if (m_hide) return;
   Docbook_DB(("(writeLineNumber)\n"));
   m_insideCodeLine = TRUE;
   if (Config_getBool(SOURCE_BROWSER))
@@ -1178,17 +1218,17 @@ DB_GEN_C
   m_t << "</entry></row>\n";
 }
 
-void DocbookGenerator::startDescTable(const QCString &title)
+void DocbookGenerator::startDescTable(const QCString &title,const bool hasInits)
 {
 DB_GEN_C
-  int ncols = 2;
+  int ncols = (hasInits?3:2);
   m_t << "<informaltable frame=\"all\">\n";
   if (!title.isEmpty()) m_t << "<title>" << convertToDocBook(title) << "</title>\n";
   m_t << "    <tgroup cols=\"" << ncols << "\" align=\"left\" colsep=\"1\" rowsep=\"1\">\n";
-  for (int i = 0; i < ncols; i++)
-  {
-    m_t << "      <colspec colname='c" << i+1 << "'/>\n";
-  }
+  int i = 1;
+  m_t << "      <colspec colname='c" << i++ << "'/>\n";
+  if (hasInits) m_t << "      <colspec colname='c" << i++ << "' align='right'/>\n";
+  m_t << "      <colspec colname='c" << i++ << "'/>\n";
   m_t << "<tbody>\n";
   m_descTable = TRUE;
 }
@@ -1225,6 +1265,17 @@ void DocbookGenerator::endDescTableTitle()
 DB_GEN_C
 }
 
+void DocbookGenerator::startDescTableInit()
+{
+DB_GEN_C
+  m_t << "</entry><entry>";
+}
+
+void DocbookGenerator::endDescTableInit()
+{
+DB_GEN_C
+}
+
 void DocbookGenerator::startDescTableData()
 {
 DB_GEN_C
@@ -1243,7 +1294,7 @@ DB_GEN_C
 void DocbookGenerator::endGroupCollaboration(DotGroupCollaboration &g)
 {
 DB_GEN_C
-  g.writeGraph(m_t,GOF_BITMAP,EOF_DocBook,dir(),fileName(),relPath,FALSE);
+  g.writeGraph(m_t,GraphOutputFormat::BITMAP,EmbeddedOutputFormat::DocBook,dir(),fileName(),relPath,FALSE);
 }
 void DocbookGenerator::startDotGraph()
 {
@@ -1252,7 +1303,7 @@ DB_GEN_C
 void DocbookGenerator::endDotGraph(DotClassGraph &g)
 {
 DB_GEN_C
-  g.writeGraph(m_t,GOF_BITMAP,EOF_DocBook,dir(),fileName(),relPath,TRUE,FALSE);
+  g.writeGraph(m_t,GraphOutputFormat::BITMAP,EmbeddedOutputFormat::DocBook,dir(),fileName(),relPath,TRUE,FALSE);
 }
 void DocbookGenerator::startInclDepGraph()
 {
@@ -1261,7 +1312,7 @@ DB_GEN_C
 void DocbookGenerator::endInclDepGraph(DotInclDepGraph &g)
 {
 DB_GEN_C
-  QCString fn = g.writeGraph(m_t,GOF_BITMAP,EOF_DocBook,dir(),fileName(),relPath,FALSE);
+  QCString fn = g.writeGraph(m_t,GraphOutputFormat::BITMAP,EmbeddedOutputFormat::DocBook,dir(),fileName(),relPath,FALSE);
 }
 void DocbookGenerator::startCallGraph()
 {
@@ -1270,7 +1321,7 @@ DB_GEN_C
 void DocbookGenerator::endCallGraph(DotCallGraph &g)
 {
 DB_GEN_C
-  QCString fn = g.writeGraph(m_t,GOF_BITMAP,EOF_DocBook,dir(),fileName(),relPath,FALSE);
+  QCString fn = g.writeGraph(m_t,GraphOutputFormat::BITMAP,EmbeddedOutputFormat::DocBook,dir(),fileName(),relPath,FALSE);
 }
 void DocbookGenerator::startDirDepGraph()
 {
@@ -1279,7 +1330,7 @@ DB_GEN_C
 void DocbookGenerator::endDirDepGraph(DotDirDeps &g)
 {
 DB_GEN_C
-  QCString fn = g.writeGraph(m_t,GOF_BITMAP,EOF_DocBook,dir(),fileName(),relPath,FALSE);
+  QCString fn = g.writeGraph(m_t,GraphOutputFormat::BITMAP,EmbeddedOutputFormat::DocBook,dir(),fileName(),relPath,FALSE);
 }
 void DocbookGenerator::startMemberDocList()
 {
