@@ -20,7 +20,6 @@
 #include "settings.h"
 #include "message.h"
 
-#if USE_SQLITE3
 
 #include "sqlite3gen.h"
 #include "doxygen.h"
@@ -30,6 +29,7 @@
 #include "util.h"
 #include "outputlist.h"
 #include "docparser.h"
+#include "docnode.h"
 #include "language.h"
 
 #include "version.h"
@@ -47,6 +47,8 @@
 #include "section.h"
 #include "fileinfo.h"
 #include "dir.h"
+#include "datetime.h"
+#include "moduledef.h"
 
 #include <sys/stat.h>
 #include <string.h>
@@ -504,9 +506,9 @@ const char * table_schema[][2] = {
 
 //////////////////////////////////////////////////////
 struct SqlStmt {
-  const char   *query = 0;
-  sqlite3_stmt *stmt = 0;
-  sqlite3 *db = 0;
+  const char   *query = nullptr;
+  sqlite3_stmt *stmt = nullptr;
+  sqlite3 *db = nullptr;
 };
 //////////////////////////////////////////////////////
 /* If you add a new statement below, make sure to add it to
@@ -518,7 +520,7 @@ SqlStmt meta_insert = {
     "( doxygen_version, schema_version, generated_at, generated_on, project_name, project_number, project_brief )"
   "VALUES "
     "(:doxygen_version,:schema_version,:generated_at,:generated_on,:project_name,:project_number,:project_brief )"
-  ,NULL
+  ,nullptr
 };
 //////////////////////////////////////////////////////
 SqlStmt incl_insert = {
@@ -526,12 +528,12 @@ SqlStmt incl_insert = {
     "( local, src_id, dst_id ) "
   "VALUES "
     "(:local,:src_id,:dst_id )"
-  ,NULL
+  ,nullptr
 };
 SqlStmt incl_select = {
   "SELECT COUNT(*) FROM includes WHERE "
   "local=:local AND src_id=:src_id AND dst_id=:dst_id"
-  ,NULL
+  ,nullptr
 };
 //////////////////////////////////////////////////////
 SqlStmt contains_insert={
@@ -539,31 +541,31 @@ SqlStmt contains_insert={
     "( inner_rowid, outer_rowid )"
   "VALUES "
     "(:inner_rowid,:outer_rowid )"
-  ,NULL
+  ,nullptr
 };
 //////////////////////////////////////////////////////
 SqlStmt path_select = {
   "SELECT rowid FROM path WHERE name=:name"
-  ,NULL
+  ,nullptr
 };
 SqlStmt path_insert = {
   "INSERT INTO path "
     "( type, local, found, name )"
   "VALUES "
     "(:type,:local,:found,:name )"
-  ,NULL
+  ,nullptr
 };
 //////////////////////////////////////////////////////
 SqlStmt refid_select =  {
   "SELECT rowid FROM refid WHERE refid=:refid"
-  ,NULL
+  ,nullptr
 };
 SqlStmt refid_insert = {
   "INSERT INTO refid "
     "( refid )"
   "VALUES "
     "(:refid )"
-  ,NULL
+  ,nullptr
 };
 //////////////////////////////////////////////////////
 SqlStmt xrefs_insert= {
@@ -571,19 +573,19 @@ SqlStmt xrefs_insert= {
     "( src_rowid, dst_rowid, context )"
   "VALUES "
     "(:src_rowid,:dst_rowid,:context )"
-  ,NULL
+  ,nullptr
 };//////////////////////////////////////////////////////
 SqlStmt reimplements_insert= {
   "INSERT INTO reimplements "
     "( memberdef_rowid, reimplemented_rowid )"
   "VALUES "
     "(:memberdef_rowid,:reimplemented_rowid )"
-  ,NULL
+  ,nullptr
 };
 //////////////////////////////////////////////////////
 SqlStmt memberdef_exists={
   "SELECT EXISTS (SELECT * FROM memberdef WHERE rowid = :rowid)"
-  ,NULL
+  ,nullptr
 };
 
 SqlStmt memberdef_incomplete={
@@ -591,7 +593,7 @@ SqlStmt memberdef_incomplete={
     "SELECT * FROM memberdef WHERE "
     "rowid = :rowid AND inline != 2 AND inline != :new_inline"
   ")"
-  ,NULL
+  ,nullptr
 };
 
 SqlStmt memberdef_insert={
@@ -713,7 +715,7 @@ SqlStmt memberdef_insert={
     ":briefdescription,"
     ":inbodydescription"
   ")"
-  ,NULL
+  ,nullptr
 };
 /*
 We have a slightly different need than the XML here. The XML can have two
@@ -732,7 +734,7 @@ SqlStmt memberdef_update_decl={
     "briefdescription = 'Declaration: ' || :briefdescription || 'Definition: ' || briefdescription,"
     "inbodydescription = 'Declaration: ' || :inbodydescription || 'Definition: ' || inbodydescription "
   "WHERE rowid = :rowid"
-  ,NULL
+  ,nullptr
 };
 SqlStmt memberdef_update_def={
   "UPDATE memberdef SET "
@@ -744,7 +746,7 @@ SqlStmt memberdef_update_def={
     "briefdescription = 'Declaration: ' || briefdescription || 'Definition: ' || :briefdescription,"
     "inbodydescription = 'Declaration: ' || inbodydescription || 'Definition: ' || :inbodydescription "
   "WHERE rowid = :rowid"
-  ,NULL
+  ,nullptr
 };
 //////////////////////////////////////////////////////
 SqlStmt member_insert={
@@ -752,7 +754,7 @@ SqlStmt member_insert={
     "( scope_rowid, memberdef_rowid, prot, virt ) "
   "VALUES "
     "(:scope_rowid,:memberdef_rowid,:prot,:virt )"
-  ,NULL
+  ,nullptr
 };
 //////////////////////////////////////////////////////
 SqlStmt compounddef_insert={
@@ -784,13 +786,13 @@ SqlStmt compounddef_insert={
     ":briefdescription,"
     ":detaileddescription"
   ")"
-  ,NULL
+  ,nullptr
 };
 SqlStmt compounddef_exists={
   "SELECT EXISTS ("
     "SELECT * FROM compounddef WHERE rowid = :rowid"
   ")"
-  ,NULL
+  ,nullptr
 };
 //////////////////////////////////////////////////////
 SqlStmt compoundref_insert={
@@ -798,7 +800,7 @@ SqlStmt compoundref_insert={
     "( base_rowid, derived_rowid, prot, virt ) "
   "VALUES "
     "(:base_rowid,:derived_rowid,:prot,:virt )"
-  ,NULL
+  ,nullptr
 };
 //////////////////////////////////////////////////////
 SqlStmt param_select = {
@@ -810,14 +812,14 @@ SqlStmt param_select = {
     "(array IS NULL OR array=:array) AND "
     "(defval IS NULL OR defval=:defval) AND "
     "(briefdescription IS NULL OR briefdescription=:briefdescription)"
-  ,NULL
+  ,nullptr
 };
 SqlStmt param_insert = {
   "INSERT INTO param "
     "( attributes, type, declname, defname, array, defval, briefdescription ) "
   "VALUES "
     "(:attributes,:type,:declname,:defname,:array,:defval,:briefdescription)"
-  ,NULL
+  ,nullptr
 };
 //////////////////////////////////////////////////////
 SqlStmt memberdef_param_insert={
@@ -825,23 +827,23 @@ SqlStmt memberdef_param_insert={
     "( memberdef_id, param_id)"
   "VALUES "
     "(:memberdef_id,:param_id)"
-  ,NULL
+  ,nullptr
 };
 
 class TextGeneratorSqlite3Impl : public TextGeneratorIntf
 {
   public:
     TextGeneratorSqlite3Impl(StringVector &l) : m_list(l) { }
-    void writeString(const QCString & /*s*/,bool /*keepSpaces*/) const
+    void writeString(std::string_view /*s*/,bool /*keepSpaces*/) const override
     {
     }
-    void writeBreak(int) const
+    void writeBreak(int) const override
     {
       DBG_CTX(("writeBreak\n"));
     }
     void writeLink(const QCString & /*extRef*/,const QCString &file,
-                   const QCString &anchor,const QCString & /*text*/
-                  ) const
+                   const QCString &anchor,std::string_view /*text*/
+                  ) const override
     {
       std::string rs = file.str();
       if (!anchor.isEmpty())
@@ -857,14 +859,14 @@ class TextGeneratorSqlite3Impl : public TextGeneratorIntf
 };
 
 
-static bool bindTextParameter(SqlStmt &s,const char *name,const QCString &value, bool _static=FALSE)
+static bool bindTextParameter(SqlStmt &s,const char *name,const QCString &value)
 {
   int idx = sqlite3_bind_parameter_index(s.stmt, name);
   if (idx==0) {
     err("sqlite3_bind_parameter_index(%s)[%s] failed: %s\n", name, s.query, sqlite3_errmsg(s.db));
     return false;
   }
-  int rv = sqlite3_bind_text(s.stmt, idx, value.data(), -1, _static==TRUE?SQLITE_STATIC:SQLITE_TRANSIENT);
+  int rv = sqlite3_bind_text(s.stmt, idx, value.data(), -1, SQLITE_TRANSIENT);
   if (rv!=SQLITE_OK) {
     err("sqlite3_bind_text(%s)[%s] failed: %s\n", name, s.query, sqlite3_errmsg(s.db));
     return false;
@@ -899,7 +901,7 @@ static int step(SqlStmt &s,bool getRowId=FALSE, bool select=FALSE)
     return -1;
   }
   if (getRowId && select) rowid = sqlite3_column_int(s.stmt, 0); // works on selects, doesn't on inserts
-  if (getRowId && !select) rowid = sqlite3_last_insert_rowid(s.db); //works on inserts, doesn't on selects
+  if (getRowId && !select) rowid = static_cast<int>(sqlite3_last_insert_rowid(s.db)); //works on inserts, doesn't on selects
   sqlite3_reset(s.stmt);
   sqlite3_clear_bindings(s.stmt); // XXX When should this really be called
   return rowid;
@@ -908,7 +910,7 @@ static int step(SqlStmt &s,bool getRowId=FALSE, bool select=FALSE)
 static int insertPath(QCString name, bool local=TRUE, bool found=TRUE, int type=1)
 {
   int rowid=-1;
-  if (name==0) return rowid;
+  if (name==nullptr) return rowid;
 
   name = stripFromPath(name);
 
@@ -928,9 +930,9 @@ static int insertPath(QCString name, bool local=TRUE, bool found=TRUE, int type=
 static void recordMetadata()
 {
   bindTextParameter(meta_insert,":doxygen_version",getFullVersion());
-  bindTextParameter(meta_insert,":schema_version","0.2.1",TRUE); //TODO: this should be a constant somewhere; not sure where
-  bindTextParameter(meta_insert,":generated_at",dateToString(TRUE));
-  bindTextParameter(meta_insert,":generated_on",dateToString(FALSE));
+  bindTextParameter(meta_insert,":schema_version","0.2.1"); //TODO: this should be a constant somewhere; not sure where
+  bindTextParameter(meta_insert,":generated_at",dateToString(DateTimeType::DateTime));
+  bindTextParameter(meta_insert,":generated_on",dateToString(DateTimeType::Date));
   bindTextParameter(meta_insert,":project_name",Config_getString(PROJECT_NAME));
   bindTextParameter(meta_insert,":project_number",Config_getString(PROJECT_NUMBER));
   bindTextParameter(meta_insert,":project_brief",Config_getString(PROJECT_BRIEF));
@@ -1026,7 +1028,7 @@ static void insertMemberFunctionParams(int memberdef_id, const MemberDef *md, co
     for (const Argument &a : declAl)
     {
       //const Argument *defArg = defAli.current();
-      const Argument *defArg = 0;
+      const Argument *defArg = nullptr;
       if (defIt!=defAl.end())
       {
         defArg = &(*defIt);
@@ -1120,14 +1122,14 @@ static void associateMember(const MemberDef *md, struct Refid member_refid, stru
 {
   // TODO: skip EnumValue only to guard against recording refids and member records
   // for enumvalues until we can support documenting them as entities.
-  if (md->memberType()==MemberType_EnumValue) return;
+  if (md->memberType()==MemberType::EnumValue) return;
   if (!md->isAnonymous()) // skip anonymous members
   {
     bindIntParameter(member_insert, ":scope_rowid", scope_refid.rowid);
     bindIntParameter(member_insert, ":memberdef_rowid", member_refid.rowid);
 
-    bindIntParameter(member_insert, ":prot", md->protection());
-    bindIntParameter(member_insert, ":virt", md->virtualness());
+    bindIntParameter(member_insert, ":prot", static_cast<int>(md->protection()));
+    bindIntParameter(member_insert, ":virt", static_cast<int>(md->virtualness()));
     step(member_insert);
   }
 }
@@ -1146,12 +1148,11 @@ static void stripQualifiers(QCString &typeStr)
 
 static int prepareStatement(sqlite3 *db, SqlStmt &s)
 {
-  int rc;
-  rc = sqlite3_prepare_v2(db,s.query,-1,&s.stmt,0);
+  int rc = sqlite3_prepare_v2(db,s.query,-1,&s.stmt,nullptr);
   if (rc!=SQLITE_OK)
   {
     err("prepare failed for:\n  %s\n  %s\n", s.query, sqlite3_errmsg(db));
-    s.db = NULL;
+    s.db = nullptr;
     return -1;
   }
   s.db = db;
@@ -1192,33 +1193,32 @@ static int prepareStatements(sqlite3 *db)
 
 static void beginTransaction(sqlite3 *db)
 {
-  char * sErrMsg = 0;
-  sqlite3_exec(db, "BEGIN TRANSACTION", NULL, NULL, &sErrMsg);
+  char * sErrMsg = nullptr;
+  sqlite3_exec(db, "BEGIN TRANSACTION", nullptr, nullptr, &sErrMsg);
 }
 
 static void endTransaction(sqlite3 *db)
 {
-  char * sErrMsg = 0;
-  sqlite3_exec(db, "END TRANSACTION", NULL, NULL, &sErrMsg);
+  char * sErrMsg = nullptr;
+  sqlite3_exec(db, "END TRANSACTION", nullptr, nullptr, &sErrMsg);
 }
 
 static void pragmaTuning(sqlite3 *db)
 {
-  char * sErrMsg = 0;
-  sqlite3_exec(db, "PRAGMA synchronous = OFF", NULL, NULL, &sErrMsg);
-  sqlite3_exec(db, "PRAGMA journal_mode = MEMORY", NULL, NULL, &sErrMsg);
-  sqlite3_exec(db, "PRAGMA temp_store = MEMORY;", NULL, NULL, &sErrMsg);
+  char * sErrMsg = nullptr;
+  sqlite3_exec(db, "PRAGMA synchronous = OFF", nullptr, nullptr, &sErrMsg);
+  sqlite3_exec(db, "PRAGMA journal_mode = MEMORY", nullptr, nullptr, &sErrMsg);
+  sqlite3_exec(db, "PRAGMA temp_store = MEMORY;", nullptr, nullptr, &sErrMsg);
 }
 
 static int initializeTables(sqlite3* db)
 {
-  int rc;
   msg("Initializing DB schema (tables)...\n");
   for (unsigned int k = 0; k < sizeof(table_schema) / sizeof(table_schema[0]); k++)
   {
     const char *q = table_schema[k][1];
-    char *errmsg;
-    rc = sqlite3_exec(db, q, NULL, NULL, &errmsg);
+    char *errmsg = nullptr;
+    int rc = sqlite3_exec(db, q, nullptr, nullptr, &errmsg);
     if (rc != SQLITE_OK)
     {
       err("failed to execute query: %s\n\t%s\n", q, errmsg);
@@ -1230,13 +1230,12 @@ static int initializeTables(sqlite3* db)
 
 static int initializeViews(sqlite3* db)
 {
-  int rc;
   msg("Initializing DB schema (views)...\n");
   for (unsigned int k = 0; k < sizeof(view_schema) / sizeof(view_schema[0]); k++)
   {
     const char *q = view_schema[k][1];
-    char *errmsg;
-    rc = sqlite3_exec(db, q, NULL, NULL, &errmsg);
+    char *errmsg = nullptr;
+    int rc = sqlite3_exec(db, q, nullptr, nullptr, &errmsg);
     if (rc != SQLITE_OK)
     {
       err("failed to execute query: %s\n\t%s\n", q, errmsg);
@@ -1274,6 +1273,31 @@ static void writeInnerClasses(const ClassLinkedRefMap &cl, struct Refid outer_re
   }
 }
 
+static void writeInnerConcepts(const ConceptLinkedRefMap &cl, struct Refid outer_refid)
+{
+  for (const auto &cd : cl)
+  {
+    struct Refid inner_refid = insertRefid(cd->getOutputFileBase());
+
+    bindIntParameter(contains_insert,":inner_rowid", inner_refid.rowid);
+    bindIntParameter(contains_insert,":outer_rowid", outer_refid.rowid);
+    step(contains_insert);
+  }
+}
+
+static void writeInnerModules(const ModuleLinkedRefMap &ml, struct Refid outer_refid)
+{
+  for (const auto &mod : ml)
+  {
+    struct Refid inner_refid = insertRefid(mod->getOutputFileBase());
+
+    bindIntParameter(contains_insert,":inner_rowid", inner_refid.rowid);
+    bindIntParameter(contains_insert,":outer_rowid", outer_refid.rowid);
+    step(contains_insert);
+  }
+}
+
+
 static void writeInnerPages(const PageLinkedRefMap &pl, struct Refid outer_refid)
 {
   for (const auto &pd : pl)
@@ -1394,27 +1418,31 @@ QCString getSQLDocBlock(const Definition *scope,
   if (doc.isEmpty()) return "";
 
   TextStream t;
-  std::unique_ptr<IDocParser> parser { createDocParser() };
-  std::unique_ptr<DocRoot>    root   {
-    validatingParseDoc(
-    *parser.get(),
-    fileName,
-    lineNr,
-    const_cast<Definition*>(scope),
-    toMemberDef(def),
-    doc,
-    FALSE,
-    FALSE,
-    0,
-    FALSE,
-    FALSE,
-    Config_getBool(MARKDOWN_SUPPORT)
-  ) };
-  XMLCodeGenerator codeGen(t);
-  // create a parse tree visitor for XML
-  auto visitor = std::make_unique<XmlDocVisitor>(t,codeGen,
-                      scope ? scope->getDefFileExtension() : QCString(""));
-  root->accept(visitor.get());
+  auto parser { createDocParser() };
+  auto ast    { validatingParseDoc(
+                *parser.get(),
+                fileName,
+                lineNr,
+                scope,
+                toMemberDef(def),
+                doc,
+                FALSE,
+                FALSE,
+                QCString(),
+                FALSE,
+                FALSE,
+                Config_getBool(MARKDOWN_SUPPORT))
+              };
+  auto astImpl = dynamic_cast<const DocNodeAST*>(ast.get());
+  if (astImpl)
+  {
+    OutputCodeList xmlCodeList;
+    xmlCodeList.add<XMLCodeGenerator>(&t);
+    // create a parse tree visitor for XML
+    XmlDocVisitor visitor(t,xmlCodeList,
+        scope ? scope->getDefFileExtension() : QCString(""));
+    std::visit(visitor,astImpl->root);
+  }
   return convertCharEntitiesToUTF8(t.str().c_str());
 }
 
@@ -1432,6 +1460,21 @@ static void getSQLDesc(SqlStmt &s,const char *col,const QCString &value,const De
     )
   );
 }
+
+static void getSQLDescCompound(SqlStmt &s,const char *col,const QCString &value,const Definition *def)
+{
+  bindTextParameter(
+    s,
+    col,
+    getSQLDocBlock(
+      def,
+      def,
+      value,
+      def->docFile(),
+      def->docLine()
+    )
+  );
+}
 ////////////////////////////////////////////
 
 /* (updated Sep 01 2018)
@@ -1448,7 +1491,7 @@ The XML schema claims:
     namespace page protocol service singleton struct type union
 
 Member kind comes from MemberDef::memberTypeName()
-  types.h defines 14 MemberType_*s
+  types.h defines 14 MemberType::*s
     _DCOP _Define _Enumeration _EnumValue _Event _Friend _Function _Interface
     _Property _Service _Signal _Slot _Typedef _Variable
       - xml doesn't include enumvalue here
@@ -1525,7 +1568,7 @@ static void generateSqlite3ForMember(const MemberDef *md, struct Refid scope_ref
   // - call graph
 
   // enum values are written as part of the enum
-  if (md->memberType()==MemberType_EnumValue) return;
+  if (md->memberType()==MemberType::EnumValue) return;
   if (md->isHidden()) return;
 
   QCString memType;
@@ -1613,7 +1656,7 @@ static void generateSqlite3ForMember(const MemberDef *md, struct Refid scope_ref
 
   bindIntParameter(memberdef_insert,":rowid", refid.rowid);
   bindTextParameter(memberdef_insert,":kind",md->memberTypeName());
-  bindIntParameter(memberdef_insert,":prot",md->protection());
+  bindIntParameter(memberdef_insert,":prot",static_cast<int>(md->protection()));
 
   bindIntParameter(memberdef_insert,":static",md->isStatic());
   bindIntParameter(memberdef_insert,":extern",md->isExternal());
@@ -1621,11 +1664,11 @@ static void generateSqlite3ForMember(const MemberDef *md, struct Refid scope_ref
   bool isFunc=FALSE;
   switch (md->memberType())
   {
-    case MemberType_Function: // fall through
-    case MemberType_Signal:   // fall through
-    case MemberType_Friend:   // fall through
-    case MemberType_DCOP:     // fall through
-    case MemberType_Slot:
+    case MemberType::Function: // fall through
+    case MemberType::Signal:   // fall through
+    case MemberType::Friend:   // fall through
+    case MemberType::DCOP:     // fall through
+    case MemberType::Slot:
       isFunc=TRUE;
       break;
     default:
@@ -1645,10 +1688,10 @@ static void generateSqlite3ForMember(const MemberDef *md, struct Refid scope_ref
     bindIntParameter(memberdef_insert,":optional",md->isOptional());
     bindIntParameter(memberdef_insert,":required",md->isRequired());
 
-    bindIntParameter(memberdef_insert,":virt",md->virtualness());
+    bindIntParameter(memberdef_insert,":virt",static_cast<int>(md->virtualness()));
   }
 
-  if (md->memberType() == MemberType_Variable)
+  if (md->memberType() == MemberType::Variable)
   {
     bindIntParameter(memberdef_insert,":mutable",md->isMutable());
     bindIntParameter(memberdef_insert,":initonly",md->isInitonly());
@@ -1669,7 +1712,7 @@ static void generateSqlite3ForMember(const MemberDef *md, struct Refid scope_ref
       bindTextParameter(memberdef_insert,":bitfield",bitfield.stripWhiteSpace());
     }
   }
-  else if (md->memberType() == MemberType_Property)
+  else if (md->memberType() == MemberType::Property)
   {
     bindIntParameter(memberdef_insert,":readable",md->isReadable());
     bindIntParameter(memberdef_insert,":writable",md->isWritable());
@@ -1695,7 +1738,7 @@ static void generateSqlite3ForMember(const MemberDef *md, struct Refid scope_ref
     bindTextParameter(memberdef_insert,":read",md->getReadAccessor());
     bindTextParameter(memberdef_insert,":write",md->getWriteAccessor());
   }
-  else if (md->memberType() == MemberType_Event)
+  else if (md->memberType() == MemberType::Event)
   {
     bindIntParameter(memberdef_insert,":addable",md->isAddable());
     bindIntParameter(memberdef_insert,":removable",md->isRemovable());
@@ -1715,11 +1758,11 @@ static void generateSqlite3ForMember(const MemberDef *md, struct Refid scope_ref
   }
 
   // + declaration/definition arg lists
-  if (md->memberType()!=MemberType_Define &&
-      md->memberType()!=MemberType_Enumeration
+  if (md->memberType()!=MemberType::Define &&
+      md->memberType()!=MemberType::Enumeration
      )
   {
-    if (md->memberType()!=MemberType_Typedef)
+    if (md->memberType()!=MemberType::Typedef)
     {
       writeMemberTemplateLists(md);
     }
@@ -1813,7 +1856,7 @@ static void generateSqlite3ForMember(const MemberDef *md, struct Refid scope_ref
   {
     insertMemberFunctionParams(memberdef_id,md,def);
   }
-  else if (md->memberType()==MemberType_Define &&
+  else if (md->memberType()==MemberType::Define &&
           !md->argsString().isEmpty())
   {
     insertMemberDefineParams(memberdef_id,md,def);
@@ -1840,13 +1883,13 @@ static void generateSqlite3Section( const Definition *d,
                       const QCString & /*header*/=QCString(),
                       const QCString & /*documentation*/=QCString())
 {
-  if (ml==0) return;
+  if (ml==nullptr) return;
   for (const auto &md : *ml)
   {
     // TODO: necessary? just tracking what xmlgen does; xmlgen says:
     // namespace members are also inserted in the file scope, but
     // to prevent this duplication in the XML output, we filter those here.
-    if (d->definitionType()!=Definition::TypeFile || md->getNamespaceDef()==0)
+    if (d->definitionType()!=Definition::TypeFile || md->getNamespaceDef()==nullptr)
     {
       generateSqlite3ForMember(md, scope_refid, d);
     }
@@ -1893,7 +1936,7 @@ static void generateSqlite3ForClass(const ClassDef *cd)
   if (cd->isReference())        return; // skip external references.
   if (cd->isHidden())           return; // skip hidden classes.
   if (cd->isAnonymous())        return; // skip anonymous compounds.
-  if (cd->templateMaster()!=0)  return; // skip generated template instances.
+  if (cd->templateMaster()!=nullptr)  return; // skip generated template instances.
 
   struct Refid refid = insertRefid(cd->getOutputFileBase());
 
@@ -1905,7 +1948,7 @@ static void generateSqlite3ForClass(const ClassDef *cd)
   bindTextParameter(compounddef_insert,":name",cd->name());
   bindTextParameter(compounddef_insert,":title",cd->title());
   bindTextParameter(compounddef_insert,":kind",cd->compoundTypeString());
-  bindIntParameter(compounddef_insert,":prot",cd->protection());
+  bindIntParameter(compounddef_insert,":prot",static_cast<int>(cd->protection()));
 
   int file_id = insertPath(cd->getDefFileName());
   bindIntParameter(compounddef_insert,":file_id",file_id);
@@ -1962,8 +2005,8 @@ static void generateSqlite3ForClass(const ClassDef *cd)
     }
   }
 
-  getSQLDesc(compounddef_insert,":briefdescription",cd->briefDescription(),cd);
-  getSQLDesc(compounddef_insert,":detaileddescription",cd->documentation(),cd);
+  getSQLDescCompound(compounddef_insert,":briefdescription",cd->briefDescription(),cd);
+  getSQLDescCompound(compounddef_insert,":detaileddescription",cd->documentation(),cd);
 
   step(compounddef_insert);
 
@@ -1974,8 +2017,8 @@ static void generateSqlite3ForClass(const ClassDef *cd)
     struct Refid derived_refid = insertRefid(cd->getOutputFileBase());
     bindIntParameter(compoundref_insert,":base_rowid", base_refid.rowid);
     bindIntParameter(compoundref_insert,":derived_rowid", derived_refid.rowid);
-    bindIntParameter(compoundref_insert,":prot",bcd.prot);
-    bindIntParameter(compoundref_insert,":virt",bcd.virt);
+    bindIntParameter(compoundref_insert,":prot",static_cast<int>(bcd.prot));
+    bindIntParameter(compoundref_insert,":virt",static_cast<int>(bcd.virt));
     step(compoundref_insert);
   }
 
@@ -1986,8 +2029,8 @@ static void generateSqlite3ForClass(const ClassDef *cd)
     struct Refid base_refid = insertRefid(cd->getOutputFileBase());
     bindIntParameter(compoundref_insert,":base_rowid", base_refid.rowid);
     bindIntParameter(compoundref_insert,":derived_rowid", derived_refid.rowid);
-    bindIntParameter(compoundref_insert,":prot",bcd.prot);
-    bindIntParameter(compoundref_insert,":virt",bcd.virt);
+    bindIntParameter(compoundref_insert,":prot",static_cast<int>(bcd.prot));
+    bindIntParameter(compoundref_insert,":virt",static_cast<int>(bcd.virt));
     step(compoundref_insert);
   }
 
@@ -2007,7 +2050,7 @@ static void generateSqlite3ForClass(const ClassDef *cd)
   // this is just a list of *local* members
   for (const auto &ml : cd->getMemberLists())
   {
-    if ((ml->listType()&MemberListType_detailedLists)==0)
+    if (!ml->listType().isDetailed())
     {
       generateSqlite3Section(cd,ml.get(),refid,"user-defined");
     }
@@ -2032,8 +2075,8 @@ static void generateSqlite3ForConcept(const ConceptDef *cd)
   bindIntParameter(compounddef_insert,":line",cd->getDefLine());
   bindIntParameter(compounddef_insert,":column",cd->getDefColumn());
 
-  getSQLDesc(compounddef_insert,":briefdescription",cd->briefDescription(),cd);
-  getSQLDesc(compounddef_insert,":detaileddescription",cd->documentation(),cd);
+  getSQLDescCompound(compounddef_insert,":briefdescription",cd->briefDescription(),cd);
+  getSQLDescCompound(compounddef_insert,":detaileddescription",cd->documentation(),cd);
 
   step(compounddef_insert);
 
@@ -2041,6 +2084,61 @@ static void generateSqlite3ForConcept(const ConceptDef *cd)
   writeTemplateList(cd);
 }
 
+static void generateSqlite3ForModule(const ModuleDef *mod)
+{
+  // + contained class definitions
+  // + contained concept definitions
+  // + member groups
+  // + normal members
+  // + brief desc
+  // + detailed desc
+  // + location (file_id, line, column)
+  // - exports
+  // + used files
+
+  if (mod->isReference() || mod->isHidden()) return;
+  struct Refid refid = insertRefid(mod->getOutputFileBase());
+  if(!refid.created && compounddefExists(refid)){return;}
+  bindIntParameter(compounddef_insert,":rowid", refid.rowid);
+  bindTextParameter(compounddef_insert,":name",mod->name());
+  bindTextParameter(compounddef_insert,":kind","module");
+
+  int file_id = insertPath(mod->getDefFileName());
+  bindIntParameter(compounddef_insert,":file_id",file_id);
+  bindIntParameter(compounddef_insert,":line",mod->getDefLine());
+  bindIntParameter(compounddef_insert,":column",mod->getDefColumn());
+
+  getSQLDescCompound(compounddef_insert,":briefdescription",mod->briefDescription(),mod);
+  getSQLDescCompound(compounddef_insert,":detaileddescription",mod->documentation(),mod);
+
+  step(compounddef_insert);
+
+  // + contained class definitions
+  writeInnerClasses(mod->getClasses(),refid);
+
+  // + contained concept definitions
+  writeInnerConcepts(mod->getConcepts(),refid);
+
+  // + member groups
+  for (const auto &mg : mod->getMemberGroups())
+  {
+    generateSqlite3Section(mod,&mg->members(),refid,"user-defined",mg->header(),
+        mg->documentation());
+  }
+
+  // + normal members
+  for (const auto &ml : mod->getMemberLists())
+  {
+    if (ml->listType().isDeclaration())
+    {
+      generateSqlite3Section(mod,ml.get(),refid,"user-defined");
+    }
+  }
+
+  // + files
+  writeInnerFiles(mod->getUsedFiles(),refid);
+}
+
 // kinds: constants library module namespace package
 static void generateSqlite3ForNamespace(const NamespaceDef *nd)
 {
@@ -2067,14 +2165,17 @@ static void generateSqlite3ForNamespace(const NamespaceDef *nd)
   bindIntParameter(compounddef_insert,":line",nd->getDefLine());
   bindIntParameter(compounddef_insert,":column",nd->getDefColumn());
 
-  getSQLDesc(compounddef_insert,":briefdescription",nd->briefDescription(),nd);
-  getSQLDesc(compounddef_insert,":detaileddescription",nd->documentation(),nd);
+  getSQLDescCompound(compounddef_insert,":briefdescription",nd->briefDescription(),nd);
+  getSQLDescCompound(compounddef_insert,":detaileddescription",nd->documentation(),nd);
 
   step(compounddef_insert);
 
   // + contained class definitions
   writeInnerClasses(nd->getClasses(),refid);
 
+  // + contained concept definitions
+  writeInnerConcepts(nd->getConcepts(),refid);
+
   // + contained namespace definitions
   writeInnerNamespaces(nd->getNamespaces(),refid);
 
@@ -2088,7 +2189,7 @@ static void generateSqlite3ForNamespace(const NamespaceDef *nd)
   // + normal members
   for (const auto &ml : nd->getMemberLists())
   {
-    if ((ml->listType()&MemberListType_declarationLists)!=0)
+    if (ml->listType().isDeclaration())
     {
       generateSqlite3Section(nd,ml.get(),refid,"user-defined");
     }
@@ -2136,8 +2237,9 @@ static void generateSqlite3ForFile(const FileDef *fd)
   for (const auto &ii : fd->includeFileList())
   {
     int src_id=insertPath(fd->absFilePath(),!fd->isReference());
-    int dst_id;
+    int dst_id=0;
     QCString dst_path;
+    bool isLocal = (ii.kind & IncludeKind_LocalMask)!=0;
 
     if(ii.fileDef) // found file
     {
@@ -2152,16 +2254,16 @@ static void generateSqlite3ForFile(const FileDef *fd)
       {
         dst_path = ii.fileDef->absFilePath();
       }
-      dst_id = insertPath(dst_path,ii.local);
+      dst_id = insertPath(dst_path,isLocal);
     }
     else // can't find file
     {
-      dst_id = insertPath(ii.includeName,ii.local,FALSE);
+      dst_id = insertPath(ii.includeName,isLocal,FALSE);
     }
 
     DBG_CTX(("-----> FileDef includeInfo for %s\n", qPrint(ii.includeName)));
-    DBG_CTX(("       local:    %d\n", ii.local));
-    DBG_CTX(("       imported: %d\n", ii.imported));
+    DBG_CTX(("       local:    %d\n", isLocal));
+    DBG_CTX(("       imported: %d\n", (ii.kind & IncludeKind_ImportMask)!=0));
     if(ii.fileDef)
     {
       DBG_CTX(("include: %s\n", qPrint(ii.fileDef->absFilePath())));
@@ -2169,11 +2271,11 @@ static void generateSqlite3ForFile(const FileDef *fd)
     DBG_CTX(("       src_id  : %d\n", src_id));
     DBG_CTX(("       dst_id: %d\n", dst_id));
 
-    bindIntParameter(incl_select,":local",ii.local);
+    bindIntParameter(incl_select,":local",isLocal);
     bindIntParameter(incl_select,":src_id",src_id);
     bindIntParameter(incl_select,":dst_id",dst_id);
     if (step(incl_select,TRUE,TRUE)==0) {
-      bindIntParameter(incl_insert,":local",ii.local);
+      bindIntParameter(incl_insert,":local",isLocal);
       bindIntParameter(incl_insert,":src_id",src_id);
       bindIntParameter(incl_insert,":dst_id",dst_id);
       step(incl_insert);
@@ -2184,8 +2286,9 @@ static void generateSqlite3ForFile(const FileDef *fd)
   for (const auto &ii : fd->includedByFileList())
   {
     int dst_id=insertPath(fd->absFilePath(),!fd->isReference());
-    int src_id;
+    int src_id=0;
     QCString src_path;
+    bool isLocal = (ii.kind & IncludeKind_LocalMask)!=0;
 
     if(ii.fileDef) // found file
     {
@@ -2200,18 +2303,18 @@ static void generateSqlite3ForFile(const FileDef *fd)
       {
         src_path = ii.fileDef->absFilePath();
       }
-      src_id = insertPath(src_path,ii.local);
+      src_id = insertPath(src_path,isLocal);
     }
     else // can't find file
     {
-      src_id = insertPath(ii.includeName,ii.local,FALSE);
+      src_id = insertPath(ii.includeName,isLocal,FALSE);
     }
 
-    bindIntParameter(incl_select,":local",ii.local);
+    bindIntParameter(incl_select,":local",isLocal);
     bindIntParameter(incl_select,":src_id",src_id);
     bindIntParameter(incl_select,":dst_id",dst_id);
     if (step(incl_select,TRUE,TRUE)==0) {
-      bindIntParameter(incl_insert,":local",ii.local);
+      bindIntParameter(incl_insert,":local",isLocal);
       bindIntParameter(incl_insert,":src_id",src_id);
       bindIntParameter(incl_insert,":dst_id",dst_id);
       step(incl_insert);
@@ -2221,6 +2324,9 @@ static void generateSqlite3ForFile(const FileDef *fd)
   // + contained class definitions
   writeInnerClasses(fd->getClasses(),refid);
 
+  // + contained concept definitions
+  writeInnerConcepts(fd->getConcepts(),refid);
+
   // + contained namespace definitions
   writeInnerNamespaces(fd->getNamespaces(),refid);
 
@@ -2234,7 +2340,7 @@ static void generateSqlite3ForFile(const FileDef *fd)
   // + normal members
   for (const auto &ml : fd->getMemberLists())
   {
-    if ((ml->listType()&MemberListType_declarationLists)!=0)
+    if (ml->listType().isDeclaration())
     {
       generateSqlite3Section(fd,ml.get(),refid,"user-defined");
     }
@@ -2282,6 +2388,12 @@ static void generateSqlite3ForGroup(const GroupDef *gd)
   // + classes
   writeInnerClasses(gd->getClasses(),refid);
 
+  // + concepts
+  writeInnerConcepts(gd->getConcepts(),refid);
+
+  // + modules
+  writeInnerModules(gd->getModules(),refid);
+
   // + namespaces
   writeInnerNamespaces(gd->getNamespaces(),refid);
 
@@ -2301,7 +2413,7 @@ static void generateSqlite3ForGroup(const GroupDef *gd)
   // + members
   for (const auto &ml : gd->getMemberLists())
   {
-    if ((ml->listType()&MemberListType_declarationLists)!=0)
+    if (ml->listType().isDeclaration())
     {
       generateSqlite3Section(gd,ml.get(),refid,"user-defined");
     }
@@ -2369,7 +2481,7 @@ static void generateSqlite3ForPage(const PageDef *pd,bool isExample)
   QCString qrefid = pd->getOutputFileBase();
   if (pd->getGroupDef())
   {
-    qrefid+=(QCString)"_"+pd->name();
+    qrefid+="_"+pd->name();
   }
   if (qrefid=="index") qrefid="indexpage"; // to prevent overwriting the generated index page.
 
@@ -2410,7 +2522,7 @@ static void generateSqlite3ForPage(const PageDef *pd,bool isExample)
   // + title
   bindTextParameter(compounddef_insert,":title",title);
 
-  bindTextParameter(compounddef_insert,":kind", isExample ? "example" : "page",TRUE);
+  bindTextParameter(compounddef_insert,":kind", isExample ? "example" : "page");
 
   int file_id = insertPath(pd->getDefFileName());
 
@@ -2433,14 +2545,13 @@ static sqlite3* openDbConnection()
 {
 
   QCString outputDirectory = Config_getString(SQLITE3_OUTPUT);
-  sqlite3 *db;
-  int rc;
+  sqlite3 *db = nullptr;
 
-  rc = sqlite3_initialize();
+  int rc = sqlite3_initialize();
   if (rc != SQLITE_OK)
   {
     err("sqlite3_initialize failed\n");
-    return NULL;
+    return nullptr;
   }
 
   std::string dbFileName = "doxygen_sqlite3.db";
@@ -2455,7 +2566,7 @@ static sqlite3* openDbConnection()
     else
     {
       err("doxygen_sqlite3.db already exists! Rename, remove, or archive it to regenerate\n");
-      return NULL;
+      return nullptr;
     }
   }
 
@@ -2463,7 +2574,7 @@ static sqlite3* openDbConnection()
     fi.absFilePath().c_str(),
     &db,
     SQLITE_OPEN_READWRITE | SQLITE_OPEN_CREATE,
-    0
+    nullptr
   );
   if (rc != SQLITE_OK)
   {
@@ -2483,17 +2594,15 @@ void generateSqlite3()
   // + related pages
   // + examples
   // + main page
-  sqlite3 *db;
-
-  db = openDbConnection();
-  if (db==NULL)
+  sqlite3 *db = openDbConnection();
+  if (db==nullptr)
   {
     return;
   }
 
 # ifdef SQLITE3_DEBUG
   // debug: show all executed statements
-  sqlite3_trace(db, &sqlLog, NULL);
+  sqlite3_trace(db, &sqlLog, nullptr);
 # endif
 
   beginTransaction(db);
@@ -2524,6 +2633,13 @@ void generateSqlite3()
     generateSqlite3ForConcept(cd.get());
   }
 
+  // + modules
+  for (const auto &mod : ModuleManager::instance().modules())
+  {
+    msg("Generating Sqlite3 output for module %s\n",qPrint(mod->name()));
+    generateSqlite3ForModule(mod.get());
+  }
+
   // + namespaces
   for (const auto &nd : *Doxygen::namespaceLinkedMap)
   {
@@ -2584,10 +2700,4 @@ void generateSqlite3()
   endTransaction(db);
 }
 
-#else // USE_SQLITE3
-void generateSqlite3()
-{
-  err("sqlite3 support has not been compiled in!\n");
-}
-#endif
 // vim: noai:ts=2:sw=2:ss=2:expandtab
